SystemOrganization addCategory: #'Kiwi-Core'!

WriteStream subclass: #IndentStream
	instanceVariableNames: 'indentLevel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!IndentStream commentStamp: 'NahuelGarbezza 1/4/2012 14:38' prior: 0!
I represent a Stream which remembers an indentation level, so cr's will indent properly. See indent and indentBack in 'indentation' category.!

!IndentStream class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 1/10/2012 17:30'!
on: aCollection

	^(super on: aCollection) initialize! !

!IndentStream class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 1/10/2012 17:37'!
with: aCollection

	^(super with: aCollection) initialize! !

!IndentStream methodsFor: 'character writing' stamp: 'NahuelGarbezza 2/21/2012 11:56'!
cr

	super cr.
	self tab: self indentLevel.! !

!IndentStream methodsFor: 'character writing' stamp: 'NahuelGarbezza 8/23/2011 10:29'!
crtab

	self cr.
	self indent.! !

!IndentStream methodsFor: 'indentation' stamp: 'NahuelGarbezza 8/23/2011 10:33'!
indent

	self tab.
	indentLevel := indentLevel + 1.! !

!IndentStream methodsFor: 'indentation' stamp: 'NahuelGarbezza 8/23/2011 10:47'!
indentBack

	indentLevel := indentLevel - 1.! !

!IndentStream methodsFor: 'accessing' stamp: 'NahuelGarbezza 12/22/2011 08:50'!
indentLevel

	^indentLevel! !

!IndentStream methodsFor: 'initialize-release' stamp: 'NahuelGarbezza 8/23/2011 10:27'!
initialize

	super initialize.
	indentLevel := 0.! !

Error subclass: #KiwiError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiError commentStamp: 'NahuelGarbezza 12/22/2011 08:07' prior: 0!
I represent errors in Kiwi, such as non-existent step definition classes, or non-existent or syntactically wrong features.!

!KiwiError class methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/22/2012 10:35'!
messageAt: aSymbol

	^self messages at: aSymbol ifAbsent: [ 'Unknown error' ]! !

!KiwiError class methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/22/2012 10:35'!
messages

	^{
	#inexistentStepDefinitionClass -> 'The class requested does not exist in the system'.
	
	} asDictionary! !

!KiwiError class methodsFor: 'signal' stamp: 'NahuelGarbezza 4/22/2012 10:36'!
signalFor: aSymbol

	self signal: (self messageAt: aSymbol)! !

PPCompositeParser subclass: #KiwiTagFilterGrammar
	instanceVariableNames: 'gherkinTag tagExpression orExpression andExpression notExpression primaryExpression parentheses andExpr notExpr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiTagFilterGrammar commentStamp: 'NahuelGarbezza 2/21/2012 11:51' prior: 0!
I have the grammar rules for parsing Kiwi tag expressions, such as @tag1, ~@tag1 or (@tag1 & @tag2) | @tag3.!

!KiwiTagFilterGrammar methodsFor: 'parsers' stamp: 'NahuelGarbezza 12/14/2011 15:00'!
andExpr

	^notExpression separatedBy: $& asParser trim! !

!KiwiTagFilterGrammar methodsFor: 'parsers' stamp: 'NahuelGarbezza 11/22/2011 19:00'!
andExpression

	^andExpr / notExpression! !

!KiwiTagFilterGrammar methodsFor: 'parsers' stamp: 'NahuelGarbezza 11/22/2011 19:37'!
gherkinTag

	^KiwiGherkinGrammar english tag! !

!KiwiTagFilterGrammar methodsFor: 'parsers' stamp: 'NahuelGarbezza 11/22/2011 19:34'!
notExpr

	^$~ asParser , primaryExpression! !

!KiwiTagFilterGrammar methodsFor: 'parsers' stamp: 'NahuelGarbezza 11/22/2011 19:34'!
notExpression

	^notExpr / primaryExpression! !

!KiwiTagFilterGrammar methodsFor: 'parsers' stamp: 'NahuelGarbezza 11/22/2011 18:38'!
orExpression

	^andExpression separatedBy: $| asParser trim! !

!KiwiTagFilterGrammar methodsFor: 'parsers' stamp: 'NahuelGarbezza 11/22/2011 18:41'!
parentheses

	^$( asParser trim , tagExpression , $) asParser trim! !

!KiwiTagFilterGrammar methodsFor: 'parsers' stamp: 'NahuelGarbezza 12/24/2011 11:03'!
primaryExpression

	^gherkinTag / parentheses / '' asParser! !

!KiwiTagFilterGrammar methodsFor: 'accessing' stamp: 'NahuelGarbezza 11/22/2011 18:37'!
start

	^tagExpression end! !

!KiwiTagFilterGrammar methodsFor: 'parsers' stamp: 'NahuelGarbezza 11/22/2011 18:37'!
tagExpression

	^orExpression / andExpression! !

KiwiTagFilterGrammar subclass: #KiwiTagFilterParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiTagFilterParser commentStamp: 'NahuelGarbezza 12/24/2011 11:08' prior: 0!
I create the objects that represent tag expressions in Kiwi.!

!KiwiTagFilterParser methodsFor: 'parsers' stamp: 'NahuelGarbezza 1/10/2012 17:48'!
andExpr

	^super andExpr ==> [ :tokens |
		tokens size = 1
			ifTrue: [ tokens first ]
			ifFalse: [ KiwiCompositeTagRule new
				operator: #&;
				leftRule: tokens first;
				rightRule: tokens third
			]
		]! !

!KiwiTagFilterParser methodsFor: 'parsers' stamp: 'NahuelGarbezza 1/10/2012 17:47'!
gherkinTag
	
	^super gherkinTag ==> [ :tokens | KiwiIncludeTagRule new tagName: tokens second ]! !

!KiwiTagFilterParser methodsFor: 'parsers' stamp: 'NahuelGarbezza 1/10/2012 17:47'!
notExpr

	^super notExpr ==> [ :tokens | KiwiNotTagRule new innerRule: tokens second ]! !

!KiwiTagFilterParser methodsFor: 'parsers' stamp: 'NahuelGarbezza 1/10/2012 17:48'!
orExpression

	^super orExpression ==> [ :tokens |
		tokens size = 1
			ifTrue: [ tokens first ]
			ifFalse: [ KiwiCompositeTagRule new
				operator: #|;
				leftRule: tokens first;
				rightRule: tokens third
			]
		]! !

!KiwiTagFilterParser methodsFor: 'parsers' stamp: 'NahuelGarbezza 12/14/2011 15:01'!
parentheses

	^super parentheses ==> #second! !

!KiwiTagFilterParser methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/25/2012 01:35'!
start

	^super start ==> [ :result |
		result isString
			ifTrue: [ KiwiNoTagsRule new ]
			ifFalse: [ result ]
	]! !

Object subclass: #KiwiASTNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiASTNode commentStamp: 'NahuelGarbezza 4/22/2012 09:21' prior: 0!
I'm the root class for all the AST nodes of Gherkin syntax.

All my subclasses must give an implementation of the #accept: method that allows the node to be visited.!

!KiwiASTNode methodsFor: 'visitor' stamp: 'NahuelGarbezza 4/1/2012 10:29'!
accept: aVisitor

	self subclassResponsibility! !

KiwiASTNode subclass: #KiwiCommentASTNode
	instanceVariableNames: 'lines'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiCommentASTNode commentStamp: 'NahuelGarbezza 4/22/2012 09:21' prior: 0!
I'm a node of the Gherkin AST that represents a comment. Comments are completely ignored by the runner.!

!KiwiCommentASTNode methodsFor: 'visitor' stamp: 'NahuelGarbezza 4/1/2012 10:41'!
accept: aVisitor

	aVisitor visitComment: self! !

!KiwiCommentASTNode methodsFor: 'copying' stamp: 'NahuelGarbezza 1/10/2012 17:19'!
copy

	^self class new lines: self lines copy! !

!KiwiCommentASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 9/4/2011 19:16'!
lines

	^lines! !

!KiwiCommentASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:24'!
lines: aCollection

	lines := aCollection! !

KiwiASTNode subclass: #KiwiCommentableASTNode
	instanceVariableNames: 'comment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiCommentableASTNode commentStamp: 'NahuelGarbezza 4/22/2012 09:21' prior: 0!
I represent a Gherkin AST node that accepts comments.!

KiwiCommentableASTNode subclass: #KiwiASTNodeWithTagsAndTitle
	instanceVariableNames: 'tagGroup title description'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiASTNodeWithTagsAndTitle commentStamp: 'NahuelGarbezza 4/22/2012 09:22' prior: 0!
I represent an AST Node of the Gherkin syntax that has a title, a description, and accepts tags.!

!KiwiASTNodeWithTagsAndTitle methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:01'!
description

	^description! !

!KiwiASTNodeWithTagsAndTitle methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:01'!
description: aString

	description := aString! !

!KiwiASTNodeWithTagsAndTitle methodsFor: 'testing' stamp: 'NahuelGarbezza 4/15/2012 12:50'!
hasTag: aTagName

	^self tagGroup notNil and: [ self tagGroup hasTag: aTagName ]! !

!KiwiASTNodeWithTagsAndTitle methodsFor: 'initialize-release' stamp: 'NahuelGarbezza 1/10/2012 17:25'!
initialize

	super initialize.
	title := ''.
	description := ''.! !

!KiwiASTNodeWithTagsAndTitle methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:01'!
tagGroup

	^tagGroup! !

!KiwiASTNodeWithTagsAndTitle methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:01'!
tagGroup: aTagGroup

	tagGroup := aTagGroup! !

!KiwiASTNodeWithTagsAndTitle methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:01'!
title

	^title! !

!KiwiASTNodeWithTagsAndTitle methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:02'!
title: aString

	title := aString! !

KiwiASTNodeWithTagsAndTitle subclass: #KiwiFeatureASTNode
	instanceVariableNames: 'background scenarios'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiFeatureASTNode commentStamp: 'NahuelGarbezza 4/1/2012 10:49' prior: 0!
I am a node of the Gherkin AST that represents a feature.

Features are the main elements of Gherkin, they contains zero or more scenarios or scenario outlines, and an optional background.!

!KiwiFeatureASTNode methodsFor: 'visitor' stamp: 'NahuelGarbezza 4/22/2012 10:54'!
accept: aVisitor

	aVisitor visitFeature: self! !

!KiwiFeatureASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 9/4/2011 12:55'!
background

	^background! !

!KiwiFeatureASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:05'!
background: aBackground

	background := aBackground! !

!KiwiFeatureASTNode methodsFor: 'testing' stamp: 'NahuelGarbezza 4/15/2012 08:28'!
hasTag: aTagName

	^(super hasTag: aTagName)
		or: [ self scenarios anySatisfy: [ :scenario | scenario hasTag: aTagName ] ]! !

!KiwiFeatureASTNode methodsFor: 'initialize-release' stamp: 'NahuelGarbezza 1/10/2012 17:25'!
initialize

	super initialize.
	scenarios := Array new.! !

!KiwiFeatureASTNode methodsFor: 'testing' stamp: 'NahuelGarbezza 9/25/2011 18:18'!
isBackgroundPresent

	^self background notNil! !

!KiwiFeatureASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:03'!
scenarios

	^scenarios! !

!KiwiFeatureASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:05'!
scenarios: aCollection

	scenarios := aCollection! !

KiwiASTNodeWithTagsAndTitle subclass: #KiwiScenarioASTNode
	instanceVariableNames: 'steps'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiScenarioASTNode commentStamp: 'NahuelGarbezza 4/1/2012 10:45' prior: 0!
I am a node of the Gherkin AST that represents a scenario. I have a list of steps (KiwiStepASTNode) and like features, I can have tags and comments.!

KiwiScenarioASTNode subclass: #KiwiBackgroundASTNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiBackgroundASTNode commentStamp: 'NahuelGarbezza 4/1/2012 10:47' prior: 0!
I am a node of the Gherkin AST that represents a background statement. I execute my steps before every scenario in the feature I belong.!

!KiwiBackgroundASTNode methodsFor: 'visitor' stamp: 'NahuelGarbezza 11/4/2011 07:28'!
accept: aVisitor

	aVisitor visitBackground: self

	! !

!KiwiScenarioASTNode methodsFor: 'visitor' stamp: 'NahuelGarbezza 4/22/2012 10:58'!
accept: aVisitor

	aVisitor visitScenario: self! !

!KiwiScenarioASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 9/25/2011 18:39'!
allScenarios

	^{self}! !

!KiwiScenarioASTNode methodsFor: 'querying' stamp: 'NahuelGarbezza 9/27/2011 03:00'!
getStepByTitle: stepTitle

	^self steps detect: [ :step | step body includesSubString: stepTitle ]! !

!KiwiScenarioASTNode methodsFor: 'initialize-release' stamp: 'NahuelGarbezza 1/10/2012 17:26'!
initialize

	super initialize.
	steps := Array new.! !

!KiwiScenarioASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 9/4/2011 19:14'!
steps

	^steps! !

!KiwiScenarioASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:17'!
steps: aCollection

	steps := aCollection! !

KiwiScenarioASTNode subclass: #KiwiScenarioOutlineASTNode
	instanceVariableNames: 'examplesSections scenarios'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiScenarioOutlineASTNode commentStamp: 'NahuelGarbezza 1/4/2012 14:47' prior: 0!
I am a node of the Gherkin AST that represents an Scenario Outline. Scenario Outlines are templates for defining many similar scenarios at once.

Example:

Scenario Outline: title
	This is an optional description

	Given an step with <arg1>
	When I do <arg2>
	Then I should see <arg3>
	
	Examples:
		| arg1   | arg2    | arg3    |
		| value1 | action1 | result1 |
		| value2 | action2 | result2 |
		| value3 | action3 | result3 |
		
This is equivalent to the following:

Scenario:
	Given an step with value1
	When I do action1
	Then I should see result1
	
Scenario:
	Given an step with value2
	When I do action2
	Then I should see result2
	
Scenario:
	Given an step with value3
	When I do action3
	Then I should see result3!

!KiwiScenarioOutlineASTNode methodsFor: 'visitor' stamp: 'NahuelGarbezza 9/25/2011 16:02'!
accept: aVisitor

	aVisitor visitScenarioOutline: self! !

!KiwiScenarioOutlineASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/22/2012 11:00'!
allScenarios

	^scenarios! !

!KiwiScenarioOutlineASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 9/25/2011 16:02'!
examplesSections

	^examplesSections! !

!KiwiScenarioOutlineASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:24'!
examplesSections: aCollection

	examplesSections := aCollection! !

!KiwiScenarioOutlineASTNode methodsFor: 'private' stamp: 'NahuelGarbezza 4/28/2012 19:20'!
generateScenarios

	scenarios := self examplesSections inject: OrderedCollection new into: [ :result :section |
		result addAll: (section table rows withIndexCollect: [ :row :index |
			KiwiScenarioASTNode new
				title: ('{1} (generated scenario #{2})' format: {self title. index printString.});
				description: self description;
				steps: (self generateStepsFor: row headers: section table headers)]).
		result].! !

!KiwiScenarioOutlineASTNode methodsFor: 'private' stamp: 'NahuelGarbezza 2/12/2012 10:03'!
generateStepsFor: aValuesRow headers: headers
	
	|stepsGenerated|
	stepsGenerated := Array new: self steps size.
	self steps deepCopy withIndexDo: [ :step :index |
		headers do: [ :head |
			(step hasPlaceholder: head) ifTrue: [
				step replacePlaceholder: head withValue: (aValuesRow at: index).
			]
		].
		stepsGenerated at: index put: step.
	].
	^stepsGenerated! !

!KiwiScenarioOutlineASTNode methodsFor: 'initialize-release' stamp: 'NahuelGarbezza 4/25/2012 06:41'!
initialize

	super initialize.
	examplesSections := Array new.! !

!KiwiCommentableASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 16:56'!
comment

	^comment! !

!KiwiCommentableASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 16:56'!
comment: aComment

	comment := aComment! !

KiwiCommentableASTNode subclass: #KiwiExamplesSectionASTNode
	instanceVariableNames: 'table'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiExamplesSectionASTNode commentStamp: 'NahuelGarbezza 4/22/2012 09:23' prior: 0!
I am a node of the Gherkin AST that represent the Examples section of a Scenario Outline. See KiwiScenarioOutlineASTNode class for more information.!

!KiwiExamplesSectionASTNode methodsFor: 'visitor' stamp: 'NahuelGarbezza 1/7/2012 20:53'!
accept: aVisitor

	^aVisitor visitExamplesSection: self! !

!KiwiExamplesSectionASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 9/25/2011 17:17'!
table

	^table! !

!KiwiExamplesSectionASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:16'!
table: aTable

	table := aTable! !

KiwiCommentableASTNode subclass: #KiwiStepASTNode
	instanceVariableNames: 'kind body argument'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiStepASTNode commentStamp: 'NahuelGarbezza 4/1/2012 10:48' prior: 0!
I am a node of the Gherkin AST. I represent a Step. Steps have a kind (Given, When, Then, And or But) and a body (with the content of the step). Optionally steps can have a table or string as argument (see classes KiwiTableASTNode and KiwiDocStringASTNode).!

!KiwiStepASTNode methodsFor: 'visitor' stamp: 'NahuelGarbezza 4/22/2012 11:03'!
accept: aVisitor

	aVisitor visitStep: self! !

!KiwiStepASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 9/4/2011 19:15'!
argument

	^argument! !

!KiwiStepASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 16:53'!
argument: aGherkinArgument

	argument := aGherkinArgument! !

!KiwiStepASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 9/4/2011 19:15'!
body

	^body! !

!KiwiStepASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 16:54'!
body: aString

	body := aString! !

!KiwiStepASTNode methodsFor: 'copying' stamp: 'NahuelGarbezza 1/10/2012 18:00'!
copy

	^self class new
		comment: self comment copy;
		kind: self kind copy;
		body: self body copy;
		argument: self argument copy;
		yourself! !

!KiwiStepASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/24/2012 06:48'!
getArgumentsFrom: aPragma

	| args stepRegex |
	args := OrderedCollection new.
	stepRegex := aPragma arguments first asRegex.
	stepRegex matches: self body.
	2 to: stepRegex subexpressionCount
		do: [ :n | args add: (stepRegex subexpression: n) ].
	self hasMultilineArgument ifTrue: [ args add: self argument asArgument ].	
	^args asArray! !

!KiwiStepASTNode methodsFor: 'testing' stamp: 'NahuelGarbezza 9/1/2011 22:44'!
hasMultilineArgument

	^self argument notNil! !

!KiwiStepASTNode methodsFor: 'placeholder processing' stamp: 'NahuelGarbezza 9/25/2011 17:18'!
hasPlaceholder: name

	^self body includesSubString: '<' , name , '>'! !

!KiwiStepASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 9/4/2011 19:15'!
kind

	^kind! !

!KiwiStepASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 16:54'!
kind: aString

	kind := aString! !

!KiwiStepASTNode methodsFor: 'placeholder processing' stamp: 'NahuelGarbezza 9/25/2011 18:16'!
replacePlaceholder: name withValue: value

	|splitted|
	splitted := self body splitOn: '<' , name , '>'.
	body := splitted first , value , splitted second.! !

KiwiASTNode subclass: #KiwiDocStringASTNode
	instanceVariableNames: 'content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiDocStringASTNode commentStamp: 'NahuelGarbezza 4/22/2012 09:23' prior: 0!
I am a node of the Gherkin AST which represents a literal string that can be passed as an optional argument to a step.

Example:

Given some step
	"""
	I store the contents
	of this string AS IS.
	"""!

!KiwiDocStringASTNode methodsFor: 'visitor' stamp: 'NahuelGarbezza 2/21/2012 11:58'!
accept: aVisitor

	aVisitor visitDocString: self! !

!KiwiDocStringASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/24/2012 06:48'!
asArgument

	^self content! !

!KiwiDocStringASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 12/22/2011 16:40'!
content

	^content! !

!KiwiDocStringASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 12/22/2011 16:39'!
content: aString

	content := aString.! !

!KiwiDocStringASTNode methodsFor: 'processing' stamp: 'NahuelGarbezza 2/7/2012 14:51'!
trimSpacesUsing: aCollectionOfSeparators
	"Remove the initial spaces of all lines of the content based on aCollectionOfSeparators"

	| spacesToRemove result |
	spacesToRemove := (Character cr split: aCollectionOfSeparators) last size.
	result := ''.
	self content lines do: [ :line |
		| lineProcessed currentLine spacesProcessed |
		spacesProcessed := spacesToRemove.
		lineProcessed := false.
		currentLine := line asOrderedCollection.
		line do: [ :char |
			char isSeparator & lineProcessed not & (spacesProcessed > 0) ifTrue: [
				currentLine :=  currentLine removeFirst; yourself.
				spacesProcessed := spacesProcessed - 1.
				] ifFalse: [ lineProcessed := true ]
			].
		spacesProcessed > 0
			ifTrue: [ spacesProcessed timesRepeat: [ currentLine addFirst: Character space ] ].
		result := result , currentLine , String cr.
	].
	result ifNotEmpty: [ result := result copyFrom: 1 to: result size - 1 ].
	self content: result.
	! !

KiwiASTNode subclass: #KiwiTableASTNode
	instanceVariableNames: 'data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiTableASTNode commentStamp: 'NahuelGarbezza 4/1/2012 10:32' prior: 0!
I represent a Gherkin Table, I am a node of the Gherkin AST. Tables can be used as step arguments (see KiwiStepASTNode class) or in an examples section (see KiwiExamplesSectionASTNode class).!

!KiwiTableASTNode methodsFor: 'visitor' stamp: 'NahuelGarbezza 1/4/2012 11:37'!
accept: aVisitor 

	aVisitor visitTable: self! !

!KiwiTableASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/24/2012 06:48'!
asArgument

	^self! !

!KiwiTableASTNode methodsFor: 'copying' stamp: 'NahuelGarbezza 9/25/2011 17:12'!
copy

	^self class data: data copy! !

!KiwiTableASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:22'!
data: aCollection

	| colLengths |
	colLengths := aCollection collect: #size.
	colLengths asSet size = 1 ifFalse: [
		KiwiError signal: 'All rows must have the same length (lengths: ' , colLengths printString , ')'
	].
	data := aCollection.! !

!KiwiTableASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 8/28/2011 16:51'!
headers
	
	^data first! !

!KiwiTableASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/8/2012 00:15'!
numberOfColumns

	^self headers size! !

!KiwiTableASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/4/2012 14:13'!
numberOfRows

	^self raw size! !

!KiwiTableASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 9/4/2011 19:16'!
raw

	^data! !

!KiwiTableASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 8/28/2011 17:12'!
rows

	^data copyFrom: 2 to: data size! !

!KiwiTableASTNode methodsFor: 'processing' stamp: 'NahuelGarbezza 4/1/2012 10:41'!
trimSpacesUsing: aCollectionOfSeparators! !

KiwiASTNode subclass: #KiwiTagGroupASTNode
	instanceVariableNames: 'tags'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiTagGroupASTNode commentStamp: 'NahuelGarbezza 2/21/2012 11:54' prior: 0!
I am a node of the Gherkin AST. I represent a group of tags, used to filter execution of scenarios and/or features.!

!KiwiTagGroupASTNode methodsFor: 'visitor' stamp: 'NahuelGarbezza 4/1/2012 10:42'!
accept: aVisitor

	aVisitor visitTagGroup: self! !

!KiwiTagGroupASTNode methodsFor: 'testing' stamp: 'NahuelGarbezza 4/15/2012 08:14'!
hasTag: aTag

	^self tags includes: aTag! !

!KiwiTagGroupASTNode methodsFor: 'initialize-release' stamp: 'NahuelGarbezza 1/10/2012 17:27'!
initialize

	super initialize.
	tags := Array new.! !

!KiwiTagGroupASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 9/4/2011 19:16'!
tags

	^tags! !

!KiwiTagGroupASTNode methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 16:42'!
tags: aCollection

	tags := aCollection! !

Object subclass: #KiwiFailingFeature
	instanceVariableNames: 'failure'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiFailingFeature commentStamp: 'NahuelGarbezza 4/22/2012 09:27' prior: 0!
I represent an invalid feature (with syntax error/s). I'm a wrapper of PPFailure (PetitParser's failure object).!

!KiwiFailingFeature methodsFor: 'visitor' stamp: 'NahuelGarbezza 2/21/2012 11:54'!
accept: aVisitor! !

!KiwiFailingFeature methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/4/2012 14:32'!
failure

	^failure! !

!KiwiFailingFeature methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/4/2012 14:31'!
failure: aPPFailure

	failure := aPPFailure! !

!KiwiFailingFeature methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/8/2012 07:22'!
scenarios

	^#()! !

!KiwiFailingFeature methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/8/2012 07:21'!
title

	^self failure printString! !

Object subclass: #KiwiFeature
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiFeature commentStamp: 'NahuelGarbezza 12/22/2011 08:08' prior: 0!
I am the model of a feature in Kiwi. KiwiRunner interacts with me.!

!KiwiFeature methodsFor: 'accessing' stamp: 'NahuelGarbezza 9/25/2011 20:24'!
featureContent

	^self subclassResponsibility! !

!KiwiFeature methodsFor: 'accessing' stamp: 'NahuelGarbezza 10/9/2011 06:27'!
featureContent: aString

	^self subclassResponsibility! !

!KiwiFeature methodsFor: 'accessing' stamp: 'NahuelGarbezza 10/9/2011 06:46'!
featureName

	^self subclassResponsibility! !

!KiwiFeature methodsFor: 'testing' stamp: 'NahuelGarbezza 2/6/2012 10:12'!
isNewFeature

	^false! !

!KiwiFeature methodsFor: 'conversion' stamp: 'NahuelGarbezza 2/5/2012 10:23'!
saveToFile: aFilePath

	^(KiwiFeatureFromFile path: aFilePath)
		featureContent: self featureContent;
		yourself! !

!KiwiFeature methodsFor: 'conversion' stamp: 'NahuelGarbezza 2/5/2012 10:30'!
saveToMethod: aMethodSymbol class: aClassSymbol

	^self
		saveToMethod: aMethodSymbol
		class: aClassSymbol
		category: 'as yet unclassified'! !

!KiwiFeature methodsFor: 'conversion' stamp: 'NahuelGarbezza 2/5/2012 10:29'!
saveToMethod: aMethodSymbol class: aClassSymbol category: category

	(Smalltalk at: aClassSymbol)
		compileSilently: aMethodSymbol asString, ' ^'''''
		classified: category.
	^(KiwiFeatureFromMethod class: aClassSymbol method: aMethodSymbol)
		featureContent: self featureContent;
		yourself! !

KiwiFeature subclass: #KiwiFeatureFromFile
	instanceVariableNames: 'path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiFeatureFromFile commentStamp: 'NahuelGarbezza 12/22/2011 08:18' prior: 0!
I am a feature model that accesses to a feature stored in a file.

Instance Variables:
	path	<ProtoObject | PseudoContext>!

!KiwiFeatureFromFile class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 1/10/2012 15:50'!
allFeaturesFromDirectory: aDirectoryPath
	"Returns a collection of features assuming all the files in the directory path are features"
	
	^(FileDirectory on: aDirectoryPath) fileNames
		collect: [ : filePath | self path: aDirectoryPath , '/' , filePath ]! !

!KiwiFeatureFromFile class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 1/10/2012 16:41'!
path: aPath

	^self new path: aPath! !

!KiwiFeatureFromFile methodsFor: 'accessing' stamp: 'NahuelGarbezza 10/27/2011 00:08'!
featureContent

	[^FileStream readOnlyFileNamed: self path do: [ :stream | stream contents ]]
		on: FileDoesNotExistException do: [ :exc | KiwiError signal: 'The file ' , self path , ' does not exist']! !

!KiwiFeatureFromFile methodsFor: 'accessing' stamp: 'NahuelGarbezza 10/27/2011 00:08'!
featureContent: aString

	[^FileStream fileNamed: self path do: [ :stream | stream nextPutAll: aString ]]
		on: FileDoesNotExistException
		do: [ :exc | KiwiError signal: 'The file ' , self path , ' does not exist']! !

!KiwiFeatureFromFile methodsFor: 'accessing' stamp: 'NahuelGarbezza 10/28/2011 10:56'!
featureName

	^('/' split: self path) last! !

!KiwiFeatureFromFile methodsFor: 'accessing' stamp: 'NahuelGarbezza 9/25/2011 22:58'!
path

	^path! !

!KiwiFeatureFromFile methodsFor: 'accessing' stamp: 'NahuelGarbezza 9/25/2011 23:04'!
path: aPath

	path := aPath! !

KiwiFeature subclass: #KiwiFeatureFromMethod
	instanceVariableNames: 'featureClass featureMethod'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiFeatureFromMethod commentStamp: 'NahuelGarbezza 12/22/2011 08:19' prior: 0!
I am a feature model that accesses a feature stored in a method in the Image.

Instance Variables:
	featureClass	<Object>
	featureMethod	<AbstractSound | Announcement class | Collection | Exception class | ExceptionSet | PPParser>!

!KiwiFeatureFromMethod class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 1/10/2012 16:41'!
class: aClassName method: aMethodName

	^self new featureClass: aClassName; featureMethod: aMethodName; yourself! !

!KiwiFeatureFromMethod methodsFor: 'accessing' stamp: 'NahuelGarbezza 10/9/2011 06:33'!
featureClass

	^featureClass! !

!KiwiFeatureFromMethod methodsFor: 'accessing' stamp: 'NahuelGarbezza 10/9/2011 06:33'!
featureClass: anObject

	featureClass := anObject! !

!KiwiFeatureFromMethod methodsFor: 'accessing' stamp: 'NahuelGarbezza 2/6/2012 17:07'!
featureContent

	[ |result|
	result := (Smalltalk at: self featureClass) new perform: self featureMethod.
	result isString ifFalse: [ Error signal ].
	^result ]
		on: Error
		do: [ :e | KiwiError
			signal: 'The method selected does not seem to contain a feature'
		]! !

!KiwiFeatureFromMethod methodsFor: 'accessing' stamp: 'bastiankruck 6/24/2012 22:36'!
featureContent: aString

	^(Smalltalk at: self featureClass)
		compileSilently: ('{1} <feature> ^''{2}''' format: {self featureMethod asString. aString.})
		classified: ((Smalltalk at: self featureClass) >> self featureMethod) category! !

!KiwiFeatureFromMethod methodsFor: 'accessing' stamp: 'NahuelGarbezza 10/9/2011 06:33'!
featureMethod

	^featureMethod! !

!KiwiFeatureFromMethod methodsFor: 'accessing' stamp: 'NahuelGarbezza 10/9/2011 06:33'!
featureMethod: anObject

	featureMethod := anObject! !

!KiwiFeatureFromMethod methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/28/2012 19:28'!
featureName

	^'{1}>>{2}' format: {self featureClass asString. self featureMethod asString}! !

KiwiFeature subclass: #KiwiNewFeature
	instanceVariableNames: 'featureContent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiNewFeature commentStamp: 'NahuelGarbezza 12/22/2011 08:19' prior: 0!
I am a feature model that represents a recently created feature, not stored yet.

Instance Variables:
	featureContent	<ProtoObject | PseudoContext>!

!KiwiNewFeature class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 9/25/2011 23:00'!
content: aString

	^super new featureContent: aString! !

!KiwiNewFeature class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 9/25/2011 23:06'!
new

	^self error: 'Use >>content: instead'! !

!KiwiNewFeature methodsFor: 'accessing' stamp: 'NahuelGarbezza 9/25/2011 23:01'!
featureContent

	^featureContent! !

!KiwiNewFeature methodsFor: 'accessing' stamp: 'NahuelGarbezza 11/2/2011 12:41'!
featureContent: aString

	featureContent := aString asString! !

!KiwiNewFeature methodsFor: 'accessing' stamp: 'NahuelGarbezza 10/28/2011 09:57'!
featureName

	^'New feature'! !

!KiwiNewFeature methodsFor: 'testing' stamp: 'NahuelGarbezza 2/6/2012 10:12'!
isNewFeature

	^true! !

Object subclass: #KiwiFormatterVisitor
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiFormatterVisitor commentStamp: 'NahuelGarbezza 1/4/2012 14:36' prior: 0!
I am responsible of write a feature in a stream in a nice, organized, formatted way.!

!KiwiFormatterVisitor methodsFor: 'actions' stamp: 'NahuelGarbezza 9/4/2011 13:02'!
formatFeatures: features

	^self formatFeatures: features on: self stream! !

!KiwiFormatterVisitor methodsFor: 'actions' stamp: 'NahuelGarbezza 9/4/2011 13:02'!
formatFeatures: features on: anIndentStream

	stream := anIndentStream.
	
	features do: [ :feature |
		feature accept: self.
		features last = feature ifFalse: [ self stream cr; cr ]
	].

	^self stream! !

!KiwiFormatterVisitor methodsFor: 'initialize-release' stamp: 'NahuelGarbezza 9/25/2011 23:08'!
initialize

	stream := IndentStream on: ''.! !

!KiwiFormatterVisitor methodsFor: 'accessing' stamp: 'NahuelGarbezza 9/25/2011 23:08'!
stream

	^stream! !

!KiwiFormatterVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 11/4/2011 07:35'!
visitBackground: aBackground

	self visitElement: aBackground withSteps: 'Background: '! !

!KiwiFormatterVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 2/21/2012 11:58'!
visitDocString: aDocString

	self stream crtab; nextPutAll: '"""'; cr.
	aDocString content lines do: [ :line | self stream nextPutAll: line; cr ].
	self stream nextPutAll: '"""'; indentBack.! !

!KiwiFormatterVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 1/4/2012 15:12'!
visitElement: elem withSteps: kw

	| isNecessaryToIndent |
	self stream nextPutAll: kw; nextPutAll: elem title.
	isNecessaryToIndent := elem description notEmpty | elem steps notEmpty.
	isNecessaryToIndent ifTrue: [ self stream crtab ].
	elem description ifNotEmpty: [ self stream nextPutAll: elem description ].
	elem steps ifNotEmpty: [
		self stream cr.
		elem description ifNotEmpty: [ self stream cr ].
	].
	elem steps do: [ :step |
		step accept: self.
		elem steps last = step ifFalse: [ self stream cr ].
	].
	isNecessaryToIndent ifTrue: [ self stream indentBack ].! !

!KiwiFormatterVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 1/8/2012 00:16'!
visitExamplesSection: anExamplesSection

	self stream nextPutAll: 'Examples:'.
	anExamplesSection table accept: self.! !

!KiwiFormatterVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 1/9/2012 23:44'!
visitFeature: aFeature

	aFeature tagGroup ifNotNil: [ aFeature tagGroup accept: self ].
	self stream
		nextPutAll: 'Feature: '; nextPutAll: aFeature title; crtab;
		nextPutAll: aFeature description.
	aFeature scenarios ifNotEmpty: [ 
		self stream cr.
		aFeature description ifNotEmpty: [ self stream cr ]
	].
	aFeature isBackgroundPresent ifTrue: [
		aFeature background accept: self.
		self stream cr; cr.
	].
	aFeature scenarios do: [ :elem |
		elem accept: self.
		aFeature scenarios last = elem ifFalse: [ self stream cr; cr ].
	]! !

!KiwiFormatterVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 1/9/2012 23:42'!
visitScenario: aScenario

	aScenario tagGroup ifNotNil: [ aScenario tagGroup accept: self ].
	self visitElement: aScenario withSteps: 'Scenario: '! !

!KiwiFormatterVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 1/8/2012 00:22'!
visitScenarioOutline: aScenarioOutline
	
	self visitScenario: aScenarioOutline.
	
	aScenarioOutline examplesSections do: [ :section |
		section = aScenarioOutline examplesSections first
			ifTrue: [ self stream crtab ] ifFalse: [ self stream cr ].
		self stream cr.
		section accept: self.
	].
	self stream indentBack.! !

!KiwiFormatterVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 4/24/2012 22:23'!
visitStep: aStep
	
	self stream nextPutAll: aStep kind; space; nextPutAll: aStep body.
	aStep hasMultilineArgument ifTrue: [ aStep argument accept: self ].! !

!KiwiFormatterVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 1/10/2012 15:28'!
visitTable: aTable

	| maxWidths |
	self stream crtab.
	maxWidths := aTable headers collectWithIndex: [ :head :i |
		(aTable raw collect: [ :row | (row at: i) size ]) max
	].
	aTable raw do: [ :row |
		self stream nextPut: $|.
		row doWithIndex: [ :cell :i |
			self stream
				nextPut: Character space;
				nextPutAll: cell;
				nextPutAll: (Array new: (maxWidths at: i) - cell size withAll: Character space);
				nextPut: Character space;
				nextPut: $|
		].
		row = aTable raw last ifFalse: [ self stream cr ].
	].
	self stream indentBack.! !

!KiwiFormatterVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 2/12/2012 10:14'!
visitTagGroup: aTagGroup 

	aTagGroup tags do: [ :tag |
		self stream nextPut: $@; nextPutAll: tag; cr
	]! !

Object subclass: #KiwiGherkinGrammar
	instanceVariableNames: 'language'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiGherkinGrammar commentStamp: 'NahuelGarbezza 1/4/2012 14:36' prior: 0!
I have the grammar rules of Gherkin syntax. Gherkin is the language used to write Kiwi features.!

!KiwiGherkinGrammar class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 12/14/2011 14:57'!
detectLanguageFor: featureContent

	(featureContent beginsWith: self languageDeclaration)
		ifTrue: [ | lang |
			lang := ('\s' asRegex split: (self languageDeclaration split: featureContent) second) first asSymbol.
			(KiwiInternationalization supportLanguage: lang)
				ifTrue: [^lang]
				ifFalse: [KiwiError signal: 'Language "' , lang asString , '" is not supported by Kiwi']]
		ifFalse: [ ^KiwiInternationalization defaultLanguage ]

! !

!KiwiGherkinGrammar class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 10/28/2011 09:18'!
detectingLanguage: featureContent

	^self language: (self detectLanguageFor: featureContent)! !

!KiwiGherkinGrammar class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 12/14/2011 14:57'!
english
	
	^self language: #en! !

!KiwiGherkinGrammar class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 10/26/2011 20:00'!
language: languageSymbol
	
	^super new language: languageSymbol! !

!KiwiGherkinGrammar class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 10/28/2011 09:17'!
languageDeclaration

	^'# language: '! !

!KiwiGherkinGrammar class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 10/26/2011 19:59'!
new

	^self error: 'Do not use this constructor. Use language specific constructors, like >>english or the generic constructor >>language:'! !

!KiwiGherkinGrammar class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 12/14/2011 14:57'!
spanish
	
	^self language: #es! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 2/12/2012 10:17'!
background
	
	^self comment trim
	, self backgroundKeyword trim
	, self untilEndOfLine
	, self untilKeyword trim
	, self steps! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 3/11/2012 15:29'!
backgroundKeyword

	^self getParserForKeywordWithColon: #background! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 12/13/2011 13:50'!
comment
	
	^self commentLine trim star! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 2/12/2012 10:17'!
commentLine
	
	^#blank asParser star
	, $# asParser
	, self untilEndOfLine! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 1/4/2012 10:50'!
docString
	
	| open close |
	open := '"""' asParser , #blank asParser star , #newline asParser.
	close := #newline asParser optional, #blank asParser star , '"""' asParser , #space asParser star.
	^ open , close negate star flatten , close! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 12/13/2011 13:53'!
examples
	
	^self comment , self examplesKeyword trim , self untilKeyword , self table trim! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 3/11/2012 15:28'!
examplesKeyword

	^self getParserForKeywordWithColon: #examples! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 10/26/2011 20:01'!
examplesSections

	^self examples star! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 2/12/2012 10:18'!
feature

	^self comment trim
	, self tagGroup trim
	, self featureKeyword , self untilEndOfLine trim
	, (self scenarioOutline / self scenario / self background)
		negate star flatten trim
	, self background optional
	, self scenarios! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 3/11/2012 15:29'!
featureKeyword

	^self getParserForKeywordWithColon: #feature! !

!KiwiGherkinGrammar methodsFor: 'private' stamp: 'NahuelGarbezza 3/11/2012 15:28'!
getKeywordsFor: aKeywordIdentifier

	|result|
	result := KiwiInternationalization
		getKeyword: aKeywordIdentifier
		fromLanguage: self language.
	^result isString ifTrue: [ result := Array with: result ] ifFalse: [ result ]! !

!KiwiGherkinGrammar methodsFor: 'private' stamp: 'NahuelGarbezza 3/11/2012 15:27'!
getParserForKeyword: aKeywordIdentifier

	^(self getKeywordsFor: aKeywordIdentifier)
		inject: PPFailingParser new
		into: [ :parser :kw | parser / kw asParser ]
! !

!KiwiGherkinGrammar methodsFor: 'private' stamp: 'NahuelGarbezza 3/11/2012 15:30'!
getParserForKeywordWithColon: aKeywordIdentifier

	^((self getKeywordsFor: aKeywordIdentifier) collect: [ :kw | kw , ':' ])
		inject: PPFailingParser new
		into: [ :parser :kw | parser / kw asParser ]! !

!KiwiGherkinGrammar methodsFor: 'accessing' stamp: 'NahuelGarbezza 10/26/2011 20:00'!
language

	^language! !

!KiwiGherkinGrammar methodsFor: 'private' stamp: 'NahuelGarbezza 10/26/2011 20:01'!
language: aLanguageSymbol

	language := aLanguageSymbol.! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 2/12/2012 10:17'!
multilineArgument
	
	^#newline asParser
	, #space asParser star
	, (self table / self docString)! !

!KiwiGherkinGrammar methodsFor: 'aux parser definitions' stamp: 'NahuelGarbezza 1/4/2012 11:02'!
printable

	^#space asParser negate! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 2/12/2012 10:16'!
reservedWordsAndSymbols
	
	^self stepKeyword
	/ self scenarioKeyword
	/ self scenarioOutlineKeyword
	/ self examplesKeyword
	/ self table
	/ self tag
	/ self commentLine! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 12/23/2011 17:16'!
scenario

	^self comment
	, self tagGroup trim
	, self scenarioKeyword , #blank asParser star , self untilEndOfLine
	, self untilKeyword trim
	, self steps! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 3/11/2012 15:29'!
scenarioKeyword

	^self getParserForKeywordWithColon: #scenario! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 12/13/2011 13:56'!
scenarioOutline

	^self comment trim
	, self tagGroup
	, self scenarioOutlineKeyword trim , self untilEndOfLine
	, self untilKeyword trim
	, self steps
	, self examplesSections trim! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 3/11/2012 15:29'!
scenarioOutlineKeyword

	^self getParserForKeywordWithColon: #scenarioOutline! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 1/4/2012 11:08'!
scenarios

	^(self scenario / self scenarioOutline) star! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 12/13/2011 13:47'!
step
	
	^self comment trim
	, self stepKeyword , #blank asParser plus
	, (self printable , self untilEndOfLine) flatten
	, self multilineArgument optional! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 3/11/2012 15:21'!
stepKeyword

	^#(given when then and but) inject: PPFailingParser new into: [ :parser :symbol | 
		parser / (self getParserForKeyword: symbol)
	]! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 1/4/2012 11:10'!
steps

	^self step star! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 2/12/2012 10:15'!
table

	| cell row |
	cell := ($| asParser / #newline asParser) negate plus flatten , $| asParser
		==> [ :tokens | tokens first trimBoth ].
	row := $| asParser , cell plus , #newline asParser optional, #space asParser star
		==> #second.
	^row plus! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 2/12/2012 10:15'!
tag

	^$@ asParser
	, ($@ asParser / $( asParser / $) asParser / #space asParser)
		negate plus flatten! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 2/12/2012 10:16'!
tagGroup
	
	^#space asParser star
	, (self tag , # space asParser plus) star! !

!KiwiGherkinGrammar methodsFor: 'aux parser definitions' stamp: 'NahuelGarbezza 10/26/2011 20:00'!
untilEndOfLine

	^#newline asParser negate star flatten! !

!KiwiGherkinGrammar methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 10/26/2011 20:14'!
untilKeyword
	
	^self reservedWordsAndSymbols negate star flatten! !

KiwiGherkinGrammar subclass: #KiwiGherkinParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiGherkinParser commentStamp: 'NahuelGarbezza 4/22/2012 09:28' prior: 0!
I am the Gherkin syntax parser. Gherkin is the language used to write Kiwi features.
I build an Abstract Syntax Tree, in order to be executed later. See all classes ending with *ASTNode for more information.!

!KiwiGherkinParser methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 4/1/2012 10:38'!
background

	^super background ==> [ :tokens |
		KiwiBackgroundASTNode new
			comment: tokens first;
			title: tokens third;
			description: tokens fourth;
			steps: tokens fifth
	]! !

!KiwiGherkinParser methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 4/1/2012 10:30'!
comment

	^super comment ==> [ :tokens | KiwiCommentASTNode new lines: tokens ]! !

!KiwiGherkinParser methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 11/22/2011 15:36'!
commentLine

	^super commentLine ==> #third! !

!KiwiGherkinParser methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 4/1/2012 10:28'!
docString

	^super docString ==> [ :tokens | KiwiDocStringASTNode new content: tokens fourth ]! !

!KiwiGherkinParser methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 4/1/2012 10:43'!
examples

	^super examples ==> [ :tokens |
		KiwiExamplesSectionASTNode new
			comment: tokens first;
			table: tokens fourth
	]! !

!KiwiGherkinParser methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 4/1/2012 10:39'!
feature

	^super feature ==> [ :tokens |
		KiwiFeatureASTNode new
			title: tokens fourth;
			description: tokens fifth;
			comment: tokens first;
			tagGroup: tokens second;
			background: tokens sixth;
			scenarios: tokens seventh.
	]! !

!KiwiGherkinParser methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 1/4/2012 10:38'!
multilineArgument

	^super multilineArgument ==> [ :tokens | 
		tokens third trimSpacesUsing: tokens second.
		tokens third.
	]! !

!KiwiGherkinParser methodsFor: 'actions' stamp: 'NahuelGarbezza 4/23/2012 14:47'!
parseFeature: aFeatureContent

	^self feature parse: aFeatureContent! !

!KiwiGherkinParser methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 4/1/2012 10:38'!
scenario

	^super scenario ==> [ :tokens |
		KiwiScenarioASTNode new
			comment: tokens first;
			tagGroup: tokens second;
			title: tokens fifth;
			description: tokens sixth trimBoth;
			steps: tokens seventh
	]! !

!KiwiGherkinParser methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 4/25/2012 06:42'!
scenarioOutline

	^super scenarioOutline ==> [ :tokens |
		KiwiScenarioOutlineASTNode new
			title: tokens fourth;
			description: tokens fifth trimBoth;
			comment: tokens first;
			tagGroup: tokens second;
			steps: tokens sixth;
			examplesSections: tokens seventh;
			generateScenarios.
	]! !

!KiwiGherkinParser methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 4/1/2012 10:37'!
step

	^super step ==> [ :tokens |
		KiwiStepASTNode new
			comment: tokens first;
			kind: tokens second;
			body: tokens fourth trimBoth;
			argument: tokens fifth
	]! !

!KiwiGherkinParser methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 4/1/2012 10:31'!
table

	^super table ==> [ :tokens | KiwiTableASTNode new data: tokens ]! !

!KiwiGherkinParser methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 11/22/2011 15:32'!
tag

	^super tag ==> #second! !

!KiwiGherkinParser methodsFor: 'parser definitions' stamp: 'NahuelGarbezza 4/1/2012 10:40'!
tagGroup

	^super tagGroup ==>
		[ :tokens | KiwiTagGroupASTNode new tags: (tokens second collect: #first) ]! !

Object subclass: #KiwiInternationalization
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiInternationalization commentStamp: 'NahuelGarbezza 5/3/2012 01:46' prior: 0!
I am responsible for Kiwi internationalization issues, such as checking keywords, get a keyword from a given language, etc. Gherkin has 10 keywords.

I store the translations as class-side methods. See 'translations' category.

The template for adding translations is the following:

<lang>

	^{
	#given -> <keyword or keywords>.
	#when -> <keyword or keywords>.
	#then -> <keyword or keywords>.
	#and -> <keyword or keywords>.
	#but -> <keyword or keywords>.
	#feature -> <keyword or keywords>.
	#scenario -> <keyword or keywords>.
	#background -> <keyword or keywords>.
	#scenarioOutline -> <keyword or keywords>.
	#examples -> <keyword or keywords>.
	}

where

	* <lang> is the language identifier, for instance en or es. It's the method name.
	* <keyword or keywords> is a String with the keyword or an Array of strings with all possible keywords.!

!KiwiInternationalization class methodsFor: 'accessing' stamp: 'NahuelGarbezza 12/22/2011 08:39'!
availableLanguages

	^self class allMethodsInCategory: #translations! !

!KiwiInternationalization class methodsFor: 'accessing' stamp: 'NahuelGarbezza 10/28/2011 09:32'!
defaultLanguage
	
	^#en! !

!KiwiInternationalization class methodsFor: 'translations' stamp: 'NahuelGarbezza 1/4/2012 11:13'!
en

	^{
	#given -> 'Given'.
	#when -> 'When'.
	#then -> 'Then'.
	#and -> 'And'.
	#but -> 'But'.
	#feature -> 'Feature'.
	#scenario -> 'Scenario'.
	#background -> 'Background'.
	#scenarioOutline -> 'Scenario Outline'.
	#examples -> #('Examples' 'Scenarios').
	}! !

!KiwiInternationalization class methodsFor: 'translations' stamp: 'NahuelGarbezza 1/4/2012 11:13'!
es

	^{
	#given -> #('Dado' 'Dada' 'Dados' 'Dadas').
	#when -> 'Cuando'.
	#then -> 'Entonces'.
	#and -> 'Y'.
	#but -> 'Pero'.
	#feature -> 'Caracterstica'.
	#scenario -> 'Escenario'.
	#background -> 'Antecedentes'.
	#scenarioOutline -> 'Esquema del escenario'.
	#examples -> #('Ejemplos' 'Escenarios').
	}! !

!KiwiInternationalization class methodsFor: 'accessing' stamp: 'NahuelGarbezza 2/7/2012 14:38'!
getKeyword: aKeywordIdentifier fromLanguage: aLanguageSymbol

	^(self perform: aLanguageSymbol) asDictionary at: aKeywordIdentifier! !

!KiwiInternationalization class methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:59'!
getKeywordIdentifier: aKeyword
	
	self availableLanguages do:  [ :lang | | kwID |
		kwID := self keywordIdentifiers detect: [ :kw | | ks |
			ks := self getKeyword: kw fromLanguage: lang.
			(ks isArray and: [ ks includes: aKeyword ]) | (ks isString and: [ ks = aKeyword ])
		] ifNone: [].
		kwID notNil ifTrue: [ ^kwID ]
	].
	KiwiError signal: 'Could not find a keyword identifier. ''' , aKeyword , ''' is not a Gherkin keyword.'! !

!KiwiInternationalization class methodsFor: 'testing' stamp: 'NahuelGarbezza 5/3/2012 01:46'!
isKeyword: aKeywordIdentifier fromLanguage: aLanguageSymbol

	self availableLanguages do: [ :lang |
		[self getKeywordIdentifier: aKeywordIdentifier. ^true] on: KiwiError do: [ :e | ]
	].
	^false! !

!KiwiInternationalization class methodsFor: 'private' stamp: 'NahuelGarbezza 10/26/2011 19:30'!
keywordIdentifiers

	^#(given when then and but feature scenario scenarioOutline background examples)! !

!KiwiInternationalization class methodsFor: 'testing' stamp: 'NahuelGarbezza 10/28/2011 09:36'!
supportLanguage: aLanguageSymbol

	^self availableLanguages includes: aLanguageSymbol! !

Object subclass: #KiwiProfile
	instanceVariableNames: 'runner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiProfile commentStamp: 'NahuelGarbezza 4/22/2012 09:30' prior: 0!
I am the base class for Kiwi profiles. All profiles should extend from me. A profile store many run configurations and it builds them easily.!

!KiwiProfile class methodsFor: 'private' stamp: 'NahuelGarbezza 12/20/2011 16:51'!
allProfilePragmas

	^Pragma allNamed: self profilePragmaName in: self! !

!KiwiProfile class methodsFor: 'accessing' stamp: 'NahuelGarbezza 12/20/2011 16:51'!
allProfiles
	
	^self allProfilePragmas collect: [ :pragma | pragma arguments first ]! !

!KiwiProfile class methodsFor: 'accessing' stamp: 'NahuelGarbezza 12/21/2011 11:37'!
profileAt: profileName

	^self new perform: (self allProfilePragmas
		detect: [ :pragma | pragma arguments first = profileName ]
		ifNone: [ KiwiError signal: 'Could not find a profile named ' , profileName ]) selector! !

!KiwiProfile class methodsFor: 'private' stamp: 'NahuelGarbezza 12/20/2011 16:18'!
profilePragmaName

	^#profile:! !

!KiwiProfile class methodsFor: 'running' stamp: 'NahuelGarbezza 12/21/2011 11:38'!
runProfile: profileName

	| runner |
	runner := (self profileAt: profileName) runner.
	runner run.
	^runner! !

!KiwiProfile methodsFor: 'configuring features' stamp: 'NahuelGarbezza 12/20/2011 16:07'!
featureFromClass: aClassName method: aMethodName

	self runner addFeature: (KiwiFeatureFromMethod class: aClassName method: aMethodName)! !

!KiwiProfile methodsFor: 'configuring features' stamp: 'NahuelGarbezza 12/20/2011 16:04'!
featureFromFile: aFilePath

	self runner addFeature: (KiwiFeatureFromFile path: aFilePath)! !

!KiwiProfile methodsFor: 'configuring features' stamp: 'NahuelGarbezza 12/20/2011 16:08'!
featuresFromClass: aClassName methods: anArray

	anArray do: [ :name | self featureFromClass: aClassName method: name ]
	! !

!KiwiProfile methodsFor: 'initialize-release' stamp: 'NahuelGarbezza 12/20/2011 16:09'!
initialize

	runner := KiwiRunner new.! !

!KiwiProfile methodsFor: 'private' stamp: 'NahuelGarbezza 12/20/2011 16:05'!
runner

	^runner! !

!KiwiProfile methodsFor: 'configuring step definitions' stamp: 'NahuelGarbezza 2/6/2012 17:35'!
stepDefinitionsClass: aSymbol
	
	self runner addStepDefinitionsClass: aSymbol.! !

!KiwiProfile methodsFor: 'configuring step definitions' stamp: 'NahuelGarbezza 2/6/2012 17:36'!
stepDefinitionsClasses: aCollection
	
	aCollection do: [ :each | self stepDefinitionsClass: each ]! !

!KiwiProfile methodsFor: 'configuring tags' stamp: 'NahuelGarbezza 12/22/2011 05:39'!
tagFilter: aString

	self runner tagFilter: aString.! !

Object subclass: #KiwiResultItem
	instanceVariableNames: 'astNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

KiwiResultItem subclass: #KiwiCompositeResult
	instanceVariableNames: 'children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiCompositeResult commentStamp: 'NahuelGarbezza 4/15/2012 08:43' prior: 0!
I represent a result that has results as children.

For instance, a feature result is a composite result because it has the results from the scenarios, and a scenario result is a composite result too, because it has the results from the steps.!

!KiwiCompositeResult methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/15/2012 08:38'!
children

	^children! !

!KiwiCompositeResult methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/24/2012 22:19'!
failedSteps

	self subclassResponsibility! !

!KiwiCompositeResult methodsFor: 'initialize-release' stamp: 'NahuelGarbezza 4/15/2012 08:38'!
initialize

	children := OrderedCollection new.! !

!KiwiCompositeResult methodsFor: 'checking status' stamp: 'NahuelGarbezza 4/29/2012 09:59'!
isFailed

	^self isPending not & self isUndefined not
		and: [ self children anySatisfy: #isFailed ]! !

!KiwiCompositeResult methodsFor: 'checking status' stamp: 'NahuelGarbezza 4/15/2012 08:36'!
isPassed

	^self children allSatisfy: #isPassed! !

!KiwiCompositeResult methodsFor: 'checking status' stamp: 'NahuelGarbezza 4/15/2012 09:12'!
isPending

	^self isUndefined not and: [ self children anySatisfy: #isPending ]! !

!KiwiCompositeResult methodsFor: 'checking status' stamp: 'NahuelGarbezza 4/15/2012 09:08'!
isUndefined
	
	^self children anySatisfy: #isUndefined! !

!KiwiCompositeResult methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/24/2012 22:19'!
passedSteps

	self subclassResponsibility! !

!KiwiCompositeResult methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/24/2012 22:19'!
pendingSteps

	self subclassResponsibility! !

!KiwiCompositeResult methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/24/2012 22:19'!
skippedSteps

	self subclassResponsibility! !

!KiwiCompositeResult methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/24/2012 22:28'!
totalSteps

	self subclassResponsibility! !

!KiwiCompositeResult methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/24/2012 22:19'!
undefinedSteps

	self subclassResponsibility! !

KiwiCompositeResult subclass: #KiwiFeatureResult
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiFeatureResult commentStamp: 'NahuelGarbezza 4/22/2012 09:25' prior: 0!
I'm a feature result of a Kiwi execution. !

!KiwiFeatureResult methodsFor: 'adding results' stamp: 'NahuelGarbezza 4/23/2012 08:27'!
addBackgroundResult: aBackgroundASTNode

	^self addScenarioResult: aBackgroundASTNode! !

!KiwiFeatureResult methodsFor: 'adding results' stamp: 'NahuelGarbezza 4/23/2012 08:29'!
addScenarioResult: aScenarioASTNode

	^self scenarioResults add: (KiwiScenarioResult new astNode: aScenarioASTNode)! !

!KiwiFeatureResult methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/8/2012 21:34'!
currentScenarioResult

	^self scenarioResults last! !

!KiwiFeatureResult methodsFor: 'count scenarios' stamp: 'NahuelGarbezza 4/22/2012 19:33'!
failedScenarios

	^(self scenarioResults select: #isFailed) size! !

!KiwiFeatureResult methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/22/2012 19:35'!
failedSteps

	^self scenarioResults
		inject: 0
		into: [ :acc :result | acc + result failedSteps ]! !

!KiwiFeatureResult methodsFor: 'count scenarios' stamp: 'NahuelGarbezza 4/22/2012 19:33'!
passedScenarios

	^(self scenarioResults select: #isPassed) size! !

!KiwiFeatureResult methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/22/2012 19:34'!
passedSteps

	^self scenarioResults
		inject: 0
		into: [ :acc :result | acc + result passedSteps ]! !

!KiwiFeatureResult methodsFor: 'count scenarios' stamp: 'NahuelGarbezza 4/22/2012 19:33'!
pendingScenarios

	^(self scenarioResults select: #isPending) size! !

!KiwiFeatureResult methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/22/2012 19:35'!
pendingSteps

	^self scenarioResults
		inject: 0
		into: [ :acc :result | acc + result pendingSteps ]! !

!KiwiFeatureResult methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/15/2012 08:38'!
scenarioResults

	^self children! !

!KiwiFeatureResult methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/22/2012 19:35'!
skippedSteps

	^self scenarioResults
		inject: 0
		into: [ :acc :result | acc + result skippedSteps ]! !

!KiwiFeatureResult methodsFor: 'count scenarios' stamp: 'NahuelGarbezza 4/22/2012 19:29'!
totalScenarios

	^self scenarioResults size! !

!KiwiFeatureResult methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/22/2012 19:34'!
totalSteps

	^self scenarioResults
		inject: 0
		into: [ :acc :result | acc + result totalSteps ]! !

!KiwiFeatureResult methodsFor: 'count scenarios' stamp: 'NahuelGarbezza 4/22/2012 19:34'!
undefinedScenarios

	^(self scenarioResults select: #isUndefined) size! !

!KiwiFeatureResult methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/22/2012 19:45'!
undefinedSteps

	^self scenarioResults
		inject: 0
		into: [ :acc :result | acc + result undefinedSteps ]! !

KiwiCompositeResult subclass: #KiwiScenarioResult
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiScenarioResult commentStamp: 'NahuelGarbezza 4/22/2012 09:25' prior: 0!
I'm a scenario result of a Kiwi execution.!

!KiwiScenarioResult methodsFor: 'adding results' stamp: 'NahuelGarbezza 4/23/2012 08:32'!
addStepResult

	^self addStepResult: nil! !

!KiwiScenarioResult methodsFor: 'adding results' stamp: 'NahuelGarbezza 4/23/2012 08:29'!
addStepResult: aStepASTNode

	^self stepResults add: (KiwiStepResult new astNode: aStepASTNode)! !

!KiwiScenarioResult methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/22/2012 19:36'!
failedSteps

	^(self stepResults select: #isFailed) size! !

!KiwiScenarioResult methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/24/2012 06:31'!
getStepByTitle: stepTitle

	^self stepResults detect: [ :sr | sr astNode body = stepTitle ] ifNone: [ nil ]
! !

!KiwiScenarioResult methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/22/2012 19:36'!
passedSteps

	^(self stepResults select: #isPassed) size! !

!KiwiScenarioResult methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/22/2012 19:36'!
pendingSteps

	^(self stepResults select: #isPending) size! !

!KiwiScenarioResult methodsFor: 'testing' stamp: 'NahuelGarbezza 4/24/2012 20:35'!
skipStepsInScenario

	^self isFailed | self isPending | self isUndefined! !

!KiwiScenarioResult methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/22/2012 19:37'!
skippedSteps

	^(self stepResults select: #isSkipped) size! !

!KiwiScenarioResult methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/15/2012 08:39'!
stepResults

	^self children! !

!KiwiScenarioResult methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/14/2012 19:34'!
totalSteps

	^self stepResults size! !

!KiwiScenarioResult methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/22/2012 19:36'!
undefinedSteps

	^(self stepResults select: #isUndefined) size! !

!KiwiResultItem methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/23/2012 08:22'!
astNode

	^astNode! !

!KiwiResultItem methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/23/2012 08:22'!
astNode: anASTNode

	astNode := anASTNode! !

!KiwiResultItem methodsFor: 'checking status' stamp: 'NahuelGarbezza 4/24/2012 22:18'!
isFailed

	self subclassResponsibility! !

!KiwiResultItem methodsFor: 'checking status' stamp: 'NahuelGarbezza 4/24/2012 22:18'!
isPassed

	self subclassResponsibility! !

!KiwiResultItem methodsFor: 'checking status' stamp: 'NahuelGarbezza 4/24/2012 22:18'!
isPending

	self subclassResponsibility! !

!KiwiResultItem methodsFor: 'checking status' stamp: 'NahuelGarbezza 4/24/2012 22:18'!
isUndefined

	self subclassResponsibility! !

KiwiResultItem subclass: #KiwiStepResult
	instanceVariableNames: 'status failure matchingMethod'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiStepResult commentStamp: 'NahuelGarbezza 4/22/2012 09:37' prior: 0!
I represent a step result of an execution. A step can have 5 possible status:

  * undefined: there was no matching definition found.
  * pending: there was a matching definition, but the method associated is tagged with the <pending> pragma.
  * failed: there was a matching definition, but when the method associated was performed, an exception has occurred.
  * passed: there was a matching definition, and the method associated was performed without errors.
  * skipped: there was a matching definition, but the method associated was not performed because a previous step in the scenario has failed (so it makes no sense to continue executing the scenario).!

!KiwiStepResult methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/8/2012 21:52'!
failure

	^failure! !

!KiwiStepResult methodsFor: 'testing' stamp: 'NahuelGarbezza 4/23/2012 08:50'!
hasMatchingDefinition

	^matchingMethod notNil! !

!KiwiStepResult methodsFor: 'checking status' stamp: 'NahuelGarbezza 4/23/2012 08:48'!
isFailed

	^status == #failed! !

!KiwiStepResult methodsFor: 'checking status' stamp: 'NahuelGarbezza 4/8/2012 22:24'!
isPassed

	^status == #passed! !

!KiwiStepResult methodsFor: 'checking status' stamp: 'NahuelGarbezza 4/14/2012 19:37'!
isPending

	^status == #pending! !

!KiwiStepResult methodsFor: 'checking status' stamp: 'NahuelGarbezza 4/8/2012 22:24'!
isSkipped

	^status == #skipped! !

!KiwiStepResult methodsFor: 'checking status' stamp: 'NahuelGarbezza 4/8/2012 22:02'!
isUndefined

	^status == #undefined! !

!KiwiStepResult methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/14/2012 19:41'!
matchingDefinition

	^matchingMethod! !

!KiwiStepResult methodsFor: 'setting status' stamp: 'NahuelGarbezza 4/23/2012 08:48'!
setFailedWith: aMethod andError: anError

	failure := anError.
	matchingMethod := aMethod.
	status := #failed.! !

!KiwiStepResult methodsFor: 'setting status' stamp: 'NahuelGarbezza 4/23/2012 08:48'!
setPassedWith: aMethod 

	matchingMethod := aMethod.
	status := #passed.! !

!KiwiStepResult methodsFor: 'setting status' stamp: 'NahuelGarbezza 4/8/2012 22:20'!
setPendingWith: aMethod 

	matchingMethod := aMethod.
	status := #pending.! !

!KiwiStepResult methodsFor: 'setting status' stamp: 'NahuelGarbezza 4/23/2012 08:48'!
setSkippedWith: aMethod 

	matchingMethod := aMethod.
	status := #skipped.! !

!KiwiStepResult methodsFor: 'setting status' stamp: 'NahuelGarbezza 4/23/2012 08:48'!
setUndefined

	status := #undefined.! !

!KiwiStepResult methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/23/2012 08:48'!
status

	^status! !

Object subclass: #KiwiResults
	instanceVariableNames: 'featureResults'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiResults commentStamp: 'NahuelGarbezza 4/8/2012 20:34' prior: 0!
I represent the results of an execution. I hold information about features, scenarios and steps, and their status (failed, passed, pending, undefined, skipped).!

!KiwiResults methodsFor: 'adding results' stamp: 'NahuelGarbezza 4/23/2012 08:30'!
addBackgroundResult: aBackgroundASTNode

	^self currentFeatureResult addBackgroundResult: aBackgroundASTNode! !

!KiwiResults methodsFor: 'adding results' stamp: 'NahuelGarbezza 4/23/2012 08:26'!
addFeatureResult: aFeatureASTNode

	^self featureResults add: (KiwiFeatureResult new astNode: aFeatureASTNode)! !

!KiwiResults methodsFor: 'adding results' stamp: 'NahuelGarbezza 4/23/2012 08:28'!
addScenarioResult: aScenarioASTNode

	^self currentFeatureResult addScenarioResult: aScenarioASTNode! !

!KiwiResults methodsFor: 'adding results' stamp: 'NahuelGarbezza 4/23/2012 08:31'!
addStepResult

	^self addStepResult: nil! !

!KiwiResults methodsFor: 'adding results' stamp: 'NahuelGarbezza 4/23/2012 08:31'!
addStepResult: aStepASTNode

	^self currentScenarioResult addStepResult: aStepASTNode! !

!KiwiResults methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/8/2012 21:31'!
currentFeatureResult

	^self featureResults last! !

!KiwiResults methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/8/2012 21:34'!
currentScenarioResult

	^self currentFeatureResult currentScenarioResult! !

!KiwiResults methodsFor: 'count features' stamp: 'NahuelGarbezza 4/22/2012 19:21'!
failedFeatures

	^(self featureResults select: #isFailed) size! !

!KiwiResults methodsFor: 'count scenarios' stamp: 'NahuelGarbezza 4/22/2012 19:30'!
failedScenarios

	^self featureResults
		inject: 0
		into: [ :acc :result | acc + result failedScenarios ]! !

!KiwiResults methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/22/2012 19:32'!
failedSteps

	^self featureResults
		inject: 0
		into: [ :acc :result | acc + result failedSteps ]! !

!KiwiResults methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/8/2012 21:30'!
featureResults

	^featureResults! !

!KiwiResults methodsFor: 'initialize-release' stamp: 'NahuelGarbezza 4/8/2012 21:29'!
initialize

	featureResults := OrderedCollection new.! !

!KiwiResults methodsFor: 'count features' stamp: 'NahuelGarbezza 4/8/2012 22:37'!
passedFeatures

	^(self featureResults select: #isPassed) size! !

!KiwiResults methodsFor: 'count scenarios' stamp: 'NahuelGarbezza 4/22/2012 19:30'!
passedScenarios

	^self featureResults
		inject: 0
		into: [ :acc :result | acc + result passedScenarios ]! !

!KiwiResults methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/22/2012 19:31'!
passedSteps

	^self featureResults
		inject: 0
		into: [ :acc :result | acc + result passedSteps ]! !

!KiwiResults methodsFor: 'count features' stamp: 'NahuelGarbezza 4/22/2012 19:29'!
pendingFeatures

	^(self featureResults select: #isPending) size! !

!KiwiResults methodsFor: 'count scenarios' stamp: 'NahuelGarbezza 4/22/2012 19:30'!
pendingScenarios

	^self featureResults
		inject: 0
		into: [ :acc :result | acc + result pendingScenarios ]! !

!KiwiResults methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/22/2012 19:32'!
pendingSteps

	^self featureResults
		inject: 0
		into: [ :acc :result | acc + result pendingSteps ]! !

!KiwiResults methodsFor: 'testing' stamp: 'NahuelGarbezza 4/8/2012 21:49'!
skipStepsInScenario
	"true if there are already a failing step in the current scenario, false otherwise"
	
	^self currentScenarioResult skipStepsInScenario! !

!KiwiResults methodsFor: 'count features' stamp: 'NahuelGarbezza 4/23/2012 08:02'!
skippedFeatures

	^0! !

!KiwiResults methodsFor: 'count scenarios' stamp: 'NahuelGarbezza 4/23/2012 08:03'!
skippedScenarios

	^0! !

!KiwiResults methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/22/2012 19:32'!
skippedSteps

	^self featureResults
		inject: 0
		into: [ :acc :result | acc + result skippedSteps ]! !

!KiwiResults methodsFor: 'count features' stamp: 'NahuelGarbezza 4/8/2012 22:26'!
totalFeatures

	^self featureResults size! !

!KiwiResults methodsFor: 'count scenarios' stamp: 'NahuelGarbezza 4/22/2012 19:28'!
totalScenarios

	^self featureResults
		inject: 0
		into: [ :acc :result | acc + result totalScenarios ]! !

!KiwiResults methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/22/2012 19:31'!
totalSteps

	^self featureResults
		inject: 0
		into: [ :acc :result | acc + result totalSteps ]! !

!KiwiResults methodsFor: 'count features' stamp: 'NahuelGarbezza 4/22/2012 19:27'!
undefinedFeatures

	^(self featureResults select: #isUndefined) size! !

!KiwiResults methodsFor: 'count scenarios' stamp: 'NahuelGarbezza 4/22/2012 19:30'!
undefinedScenarios

	^self featureResults
		inject: 0
		into: [ :acc :result | acc + result undefinedScenarios ]! !

!KiwiResults methodsFor: 'count steps' stamp: 'NahuelGarbezza 4/22/2012 19:32'!
undefinedSteps

	^self featureResults
		inject: 0
		into: [ :acc :result | acc + result undefinedSteps ]! !

Object subclass: #KiwiRunner
	instanceVariableNames: 'features stepDefinitionsClasses tagFilter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiRunner commentStamp: 'NahuelGarbezza 2/21/2012 11:42' prior: 0!
I represent a run configuration. I have the API for Kiwi users.!

!KiwiRunner methodsFor: 'actions' stamp: 'NahuelGarbezza 11/2/2011 12:44'!
addFeature: aFeatureDefinition

	self features add: aFeatureDefinition.! !

!KiwiRunner methodsFor: 'actions' stamp: 'NahuelGarbezza 12/22/2011 15:20'!
addFeatureFromClass: aClassSymbol method: aMethodSymbol

	self addFeature: (KiwiFeatureFromMethod class: aClassSymbol method: aMethodSymbol).! !

!KiwiRunner methodsFor: 'actions' stamp: 'NahuelGarbezza 2/7/2012 15:11'!
addFeatureFromDirectory: aDirectoryPath file: aFileName

	self addFeatureFromPath: ((aDirectoryPath endsWith: #/) ifTrue: [ aDirectoryPath ] ifFalse: [ aDirectoryPath , '/' ]) , aFileName! !

!KiwiRunner methodsFor: 'actions' stamp: 'NahuelGarbezza 12/22/2011 15:18'!
addFeatureFromPath: aFilePath

	self addFeature: (KiwiFeatureFromFile path: aFilePath).! !

!KiwiRunner methodsFor: 'actions' stamp: 'NahuelGarbezza 12/22/2011 15:06'!
addFeatures: aCollectionOfFeatures

	self features addAll: aCollectionOfFeatures! !

!KiwiRunner methodsFor: 'actions' stamp: 'NahuelGarbezza 12/22/2011 15:21'!
addFeaturesFromClass: aClassSymbol methods: aCollectionOfSymbols

	aCollectionOfSymbols do: [ :methodName | self addFeatureFromClass: aClassSymbol method: methodName ]! !

!KiwiRunner methodsFor: 'actions' stamp: 'NahuelGarbezza 12/22/2011 15:17'!
addFeaturesFromDirectory: aDirectoryPath files: aCollectionOfFileNames

	aCollectionOfFileNames do: [ :fileName | self addFeatureFromDirectory: aDirectoryPath file: fileName ].! !

!KiwiRunner methodsFor: 'actions' stamp: 'NahuelGarbezza 2/6/2012 09:23'!
addStepDefinitionsClass: aClassName

	self ensureClassExistence: aClassName.
	self stepDefinitionsClasses add: aClassName.! !

!KiwiRunner methodsFor: 'private' stamp: 'NahuelGarbezza 4/22/2012 10:37'!
ensureClassExistence: aClassName

	Smalltalk
		at: aClassName
		ifAbsent: [ KiwiError signalFor: #inexistentStepDefinitionClass ].! !

!KiwiRunner methodsFor: 'accessing' stamp: 'NahuelGarbezza 9/4/2011 12:48'!
features

	^features! !

!KiwiRunner methodsFor: 'initialize-release' stamp: 'NahuelGarbezza 4/23/2012 13:59'!
initialize

	features := OrderedCollection new.
	stepDefinitionsClasses := OrderedCollection new.
	tagFilter := ''.! !

!KiwiRunner methodsFor: 'profiles' stamp: 'NahuelGarbezza 12/22/2011 05:45'!
loadProfile: aProfile

	stepDefinitionsClasses := aProfile runner stepDefinitionsClasses.
	features := aProfile runner features.
	tagFilter := aProfile runner tagFilter.! !

!KiwiRunner methodsFor: 'private' stamp: 'NahuelGarbezza 4/23/2012 14:47'!
parseFeature: aFeatureContent

	| result |
	result := (KiwiGherkinParser detectingLanguage: aFeatureContent)
		parseFeature: aFeatureContent.
	^result isPetitFailure
		ifTrue: [ KiwiFailingFeature new failure: result ]
		ifFalse: [ result ]! !

!KiwiRunner methodsFor: 'private' stamp: 'NahuelGarbezza 4/23/2012 13:59'!
parsedFeatures

	^self features collect: [ :f | self parseFeature: f featureContent ]! !

!KiwiRunner methodsFor: 'accessing' stamp: 'NahuelGarbezza 2/2/2012 16:13'!
parsedTagFilter

	^KiwiTagFilterParser new parse: self tagFilter! !

!KiwiRunner methodsFor: 'actions' stamp: 'NahuelGarbezza 11/2/2011 12:44'!
removeFeature: aFeatureDefinition

	self features remove: aFeatureDefinition.! !

!KiwiRunner methodsFor: 'actions' stamp: 'NahuelGarbezza 12/22/2011 15:06'!
removeFeatures: aCollectionOfFeatures

	self features removeAll: aCollectionOfFeatures.! !

!KiwiRunner methodsFor: 'actions' stamp: 'NahuelGarbezza 2/6/2012 09:22'!
removeStepDefinitionsClass: aClassName

	self ensureClassExistence: aClassName.
	self stepDefinitionsClasses remove: aClassName.! !

!KiwiRunner methodsFor: 'running' stamp: 'NahuelGarbezza 4/23/2012 15:19'!
run

	| runner |
	runner := KiwiRunnerVisitor
		newStepDefinitionsClasses: self stepDefinitionsClasses
		tagFilter: self parsedTagFilter.
	
	self parsedFeatures do: [ :feature | feature accept: runner ].
	^runner results! !

!KiwiRunner methodsFor: 'running' stamp: 'NahuelGarbezza 4/22/2012 19:19'!
runFeature: aFeature

	(self features includes: aFeature)
		ifFalse: [ KiwiError signal: 'The feature specified is not in the runner' ]
		ifTrue: [ | oldFeatures result |
			oldFeatures := self features.
			features := OrderedCollection with: aFeature.
			result := self run.
			features := oldFeatures.
			^result
		]! !

!KiwiRunner methodsFor: 'profiles' stamp: 'NahuelGarbezza 12/22/2011 16:00'!
runner

	^self! !

!KiwiRunner methodsFor: 'accessing' stamp: 'NahuelGarbezza 9/4/2011 12:48'!
stepDefinitionsClasses

	^stepDefinitionsClasses! !

!KiwiRunner methodsFor: 'accessing' stamp: 'NahuelGarbezza 10/10/2011 14:13'!
tagFilter

	^tagFilter! !

!KiwiRunner methodsFor: 'accessing' stamp: 'NahuelGarbezza 11/22/2011 20:34'!
tagFilter: aString

	tagFilter := aString! !

Object subclass: #KiwiTagRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiTagRule commentStamp: 'NahuelGarbezza 1/8/2012 00:48' prior: 0!
I represent a tag rule in Kiwi.!

KiwiTagRule subclass: #KiwiCompositeTagRule
	instanceVariableNames: 'leftRule rightRule operator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiCompositeTagRule commentStamp: 'NahuelGarbezza 1/8/2012 00:46' prior: 0!
I represent a tag rule composed of other tag rules.!

!KiwiCompositeTagRule methodsFor: 'evaluating' stamp: 'NahuelGarbezza 1/10/2012 18:05'!
includes: aGherkinTaggedASTNode

	^(self leftRule includes: aGherkinTaggedASTNode)
		perform: self operator
		with: (self rightRule includes: aGherkinTaggedASTNode)! !

!KiwiCompositeTagRule methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:43'!
leftRule

	^leftRule! !

!KiwiCompositeTagRule methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:43'!
leftRule: anObject

	leftRule := anObject! !

!KiwiCompositeTagRule methodsFor: 'evaluating' stamp: 'NahuelGarbezza 1/10/2012 17:44'!
operator

	^operator! !

!KiwiCompositeTagRule methodsFor: 'evaluating' stamp: 'NahuelGarbezza 1/10/2012 17:44'!
operator: aSymbol

	operator := aSymbol! !

!KiwiCompositeTagRule methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:43'!
rightRule

	^rightRule! !

!KiwiCompositeTagRule methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:43'!
rightRule: anObject

	rightRule := anObject! !

KiwiTagRule subclass: #KiwiIncludeTagRule
	instanceVariableNames: 'tagName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiIncludeTagRule commentStamp: 'NahuelGarbezza 1/8/2012 00:52' prior: 0!
I represent a tag rule in Kiwi which filters all scenarios or features that have a tag name.!

!KiwiIncludeTagRule methodsFor: 'evaluating' stamp: 'NahuelGarbezza 1/9/2012 20:31'!
includes: aGherkinTaggedASTNode
	
	^aGherkinTaggedASTNode hasTag: self tagName! !

!KiwiIncludeTagRule methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:46'!
tagName

	^tagName! !

!KiwiIncludeTagRule methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:46'!
tagName: anObject

	tagName := anObject! !

KiwiTagRule subclass: #KiwiNoTagsRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiNoTagsRule commentStamp: 'NahuelGarbezza 1/9/2012 19:44' prior: 0!
I represent a 'null' tag rule (I don't filter anything)!

!KiwiNoTagsRule methodsFor: 'evaluating' stamp: 'NahuelGarbezza 1/9/2012 20:31'!
includes: aGherkinTaggedASTNode
	"Always satisfies the condition"
	
	^true! !

KiwiTagRule subclass: #KiwiNotTagRule
	instanceVariableNames: 'innerRule'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiNotTagRule commentStamp: 'NahuelGarbezza 1/8/2012 00:51' prior: 0!
I represent a tag rule in Kiwi which filters all scenarios or features that does NOT satisfy my inner tag rule. The syntax is the following: ~tagRule. Examples: ~@tag1, ~(@tag2 & @tag3)!

!KiwiNotTagRule methodsFor: 'evaluating' stamp: 'NahuelGarbezza 1/10/2012 18:05'!
includes: aGherkinTaggedASTNode

	^(self innerRule includes: aGherkinTaggedASTNode) not! !

!KiwiNotTagRule methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:46'!
innerRule

	^innerRule! !

!KiwiNotTagRule methodsFor: 'accessing' stamp: 'NahuelGarbezza 1/10/2012 17:46'!
innerRule: anObject

	innerRule := anObject! !

!KiwiTagRule methodsFor: 'evaluating' stamp: 'NahuelGarbezza 4/24/2012 22:23'!
includes: aGherkinTaggedASTNode

	"features(KiwiFeatureASTNode) or scenarios(KiwiScenarioASTNode and KiwiScenarioOutlineASTNode) can have tags"
	
	^self subclassResponsibility! !

Object subclass: #KiwiTaggedVisitor
	instanceVariableNames: 'tagFilter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiTaggedVisitor commentStamp: 'NahuelGarbezza 2/21/2012 11:51' prior: 0!
I am a visitor of the Gherkin AST, I have a tag filter to decide which nodes I have to visit. See KiwiTagRule for more information.

Instance Variables:
	tagFilter	<KiwiTagRule>!

KiwiTaggedVisitor subclass: #KiwiRunnerVisitor
	instanceVariableNames: 'currentStepKind stepDefinitions results'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kiwi-Core'!

!KiwiRunnerVisitor commentStamp: 'NahuelGarbezza 1/8/2012 00:45' prior: 0!
I travel Gherkin AST executing every node of it.!

!KiwiRunnerVisitor class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 11/28/2011 04:55'!
new

	^self error: 'Use >>newStepDefinitionsClasses: or >>newStepDefinitionsClasses: tagFilter: instead'! !

!KiwiRunnerVisitor class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 4/21/2012 03:43'!
newStepDefinitionsClasses: classes

	^super new initialize: classes! !

!KiwiRunnerVisitor class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 1/9/2012 23:32'!
newStepDefinitionsClasses: classes tagFilter: aTagFilter

	^(self newStepDefinitionsClasses: classes)	tagFilter: aTagFilter! !

!KiwiRunnerVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 4/23/2012 08:23'!
doVisitFeature: aFeature

	self results addFeatureResult: aFeature.
	aFeature scenarios do: [ :scenario |
		aFeature isBackgroundPresent ifTrue: [ aFeature background accept: self ].
		scenario accept: self.
		self restoreState.
	].! !

!KiwiRunnerVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 4/23/2012 08:26'!
doVisitScenario: aScenario

	| scenarioResult |
	scenarioResult := self results addScenarioResult: aScenario.
	self executeHooks: #before.
	aScenario steps do: [ :step | step accept: self ].
	self executeHooks: #after.
	^scenarioResult
! !

!KiwiRunnerVisitor methodsFor: 'hooks' stamp: 'NahuelGarbezza 12/14/2011 15:02'!
executeHooks: pragmaName

	stepDefinitions keysAndValuesDo: [ :className :currentInstance |
		(Pragma allNamed: pragmaName in: (Smalltalk at: className)) do: [ :pragma |
			currentInstance
				ifNil: [ (Smalltalk at: className) new perform: pragma selector ]
				ifNotNil: [ currentInstance perform: pragma selector ]
		]
	]! !

!KiwiRunnerVisitor methodsFor: 'private' stamp: 'NahuelGarbezza 11/20/2011 15:55'!
getCurrentStepInstance: className
	
	(stepDefinitions at: className)
		ifNil: [ stepDefinitions at: className put: (Smalltalk at: className) new ].
	^stepDefinitions at: className! !

!KiwiRunnerVisitor methodsFor: 'private' stamp: 'NahuelGarbezza 9/4/2011 17:15'!
getPragmaOn: className forStep: aStep

	^(Pragma allNamed: (self getStepKind: aStep) , ':' asSymbol in: (Smalltalk at: className))
		detect: [ :pragma | pragma arguments first asRegex matches: aStep body ]
		ifNone: [ nil ].! !

!KiwiRunnerVisitor methodsFor: 'private' stamp: 'NahuelGarbezza 4/24/2012 22:28'!
getStepKind: astStep

	| stepKind |
	stepKind := KiwiInternationalization getKeywordIdentifier: astStep kind.
	"And or But steps are interpreted as the previous Given, When or Then step"
	(#(and but) includes: stepKind)
		ifTrue: [ stepKind := currentStepKind ]
		ifFalse: [ currentStepKind := stepKind ].
	^stepKind! !

!KiwiRunnerVisitor methodsFor: 'initialize-release' stamp: 'NahuelGarbezza 4/21/2012 03:43'!
initStepDefinitionClasses: classNames

	stepDefinitions := (classNames collect: [ :className | className -> nil ]) asDictionary.! !

!KiwiRunnerVisitor methodsFor: 'initialize-release' stamp: 'NahuelGarbezza 4/21/2012 03:42'!
initialize: classNames

	self initStepDefinitionClasses: classNames.
	results := KiwiResults new.! !

!KiwiRunnerVisitor methodsFor: 'private' stamp: 'NahuelGarbezza 4/21/2012 03:41'!
restoreState
	"Clear all instances created to keep the state within a scenario"
	
	self
		initStepDefinitionClasses: self stepDefinitionsClasses;
		tagFilter: self tagFilter;
		results: self results.! !

!KiwiRunnerVisitor methodsFor: 'accessing' stamp: 'NahuelGarbezza 4/1/2012 19:12'!
results

	^results! !

!KiwiRunnerVisitor methodsFor: 'private' stamp: 'NahuelGarbezza 4/21/2012 03:41'!
results: aResults

	results := aResults.! !

!KiwiRunnerVisitor methodsFor: 'private' stamp: 'NahuelGarbezza 9/4/2011 18:04'!
stepDefinitionsClasses

	^stepDefinitions keys! !

!KiwiRunnerVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 4/28/2012 19:29'!
visitBackground: aBackground

	self results addBackgroundResult: aBackground.
	self visitScenario: aBackground.
! !

!KiwiRunnerVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 4/24/2012 22:20'!
visitComment: aComment! !

!KiwiRunnerVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 4/25/2012 06:45'!
visitScenarioOutline: aScenarioOutline 

	aScenarioOutline allScenarios do: [ :scenario | scenario accept: self ]! !

!KiwiRunnerVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 4/23/2012 08:35'!
visitStep: aStep

	| stepPragma matchingStepClass stepResult |
	stepResult := self results addStepResult: aStep.
	
	"Search for a step definition in all the classes with steps"
	matchingStepClass := self stepDefinitionsClasses
		detect: [ :className | (self getPragmaOn: className forStep: aStep) notNil ]
		"If there's no matches, the step is considered undefined"
		ifNone: [ ^stepResult setUndefined ].

	stepPragma := self getPragmaOn: matchingStepClass forStep: aStep.
	
	"If the step has a matching definition, but is part of a non-passed scenario, it is marked as skipped"
	self results skipStepsInScenario
		ifTrue: [ ^stepResult setSkippedWith: stepPragma method ].
	
	"If the matching step method has the <pending> pragma, is considered as pending"
	(stepPragma method pragmaAt: #pending)
		ifNotNil: [ ^stepResult setPendingWith: stepPragma method ].
	
	"The step is neither pending nor undefined, so it should be performed"
	[(self getCurrentStepInstance: matchingStepClass)
		perform: stepPragma selector
		withArguments: (aStep getArgumentsFrom: stepPragma)
	] on: Error do: [ :err | ^stepResult setFailedWith: stepPragma method andError: err ].

	"If there was no errors, the step is considered passed"
	^stepResult setPassedWith: stepPragma method! !

!KiwiTaggedVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 4/22/2012 09:50'!
doVisitFeature: aFeature

	self subclassResponsibility! !

!KiwiTaggedVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 4/22/2012 09:50'!
doVisitScenario: aScenario

	self subclassResponsibility! !

!KiwiTaggedVisitor methodsFor: 'initialize-release' stamp: 'NahuelGarbezza 1/9/2012 23:31'!
initialize

	tagFilter := KiwiNoTagsRule new.! !

!KiwiTaggedVisitor methodsFor: 'accessing' stamp: 'NahuelGarbezza 2/21/2012 11:48'!
tagFilter

	^tagFilter! !

!KiwiTaggedVisitor methodsFor: 'accessing' stamp: 'NahuelGarbezza 2/21/2012 11:48'!
tagFilter: anObject

	tagFilter := anObject! !

!KiwiTaggedVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 4/22/2012 09:50'!
visitFeature: aFeature

	(self tagFilter includes: aFeature)
		ifTrue: [ ^self doVisitFeature: aFeature ]! !

!KiwiTaggedVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 4/15/2012 15:30'!
visitScenario: aScenario

	(self tagFilter includes: aScenario)
		ifTrue: [ ^self doVisitScenario: aScenario ]! !

!KiwiTaggedVisitor methodsFor: 'visitor methods' stamp: 'NahuelGarbezza 4/22/2012 09:50'!
visitStep: aStep

	self subclassResponsibility! !
